Setup
-----

You need an installation of Python 3, Jupyter and the following packages (I used the pip package manager)

    $ pip3 install matplotlib numpy scipy pandas torch seaborn tqdm pyro-ppl numpyro

We carried the experiments out on a computer with a Intel Core i7-8700 CPU, @ 3.20 GHz x 12 and 16 GB RAM, running Ubuntu 20.04.
The exact versions of the installed packages shouldn't matter.
For reference, they were: `matplotlib-3.3.1`, `numpy-1.19.1`, `scipy-1.5.2`, `pandas-1.1.2`, `torch-1.6.0`, `seaborn-0.11.0`, `tqdm-4.48.2`, `pyro-ppl-1.4.0`, and `numpyro-0.6.0`.

Reproducing the experiments
---------------------------

To reproduce the experiments from the paper, you need to generate the samples:

1. Generate samples using Nonparametric Hamiltonian Monte Carlo (NP-DHMC).
2. Run the evaluation scripts to produce the plots and tables from the paper.

If you just want to view the results of the evaluation, you can simply view the `evaluation_*.ipynb` notebooks from step 3.
But if you want to run the evaluation yourself, you'll need to complete steps 1 and 2 first.

### Step 1: Generating the NP-DHMC samples

The model code for the 4 examples is in the `example_*.py` files.
You can simply run them as follows:

    $ python3 example_geometric.py
    $ python3 example_walk.py
    $ python3 example_gmm.py
    $ python3 example_dirichlet.py

These runs will save their results in `samples_produced/{experiment_name}_{run}_{sample_count}_{hyperparameters}.pickle`.

Note that some of those runs can take a while to complete.
Overall, the runs took over a day to complete.

### Step 2: Evaluation
The evaluation (ESS, LPPD, plots) for the experiments is in the Jupyter notebooks `evaluation_*.ipynb` files.
If you use Jupyter, you can see the saved results and don't have to run the code.
To view the notebooks, run the following:

    $ jupyter notebook evaluation_geometric.ipynb
    $ jupyter notebook evaluation_walk.ipynb
    $ jupyter notebook evaluation_gmm.ipynb
    $ jupyter notebook evaluation_dirichlet.ipynb

*Note:* If you not only want to view the notebooks but also run the code, you'll need to produce the samples for all the inference algorithms first.

When running the notebooks, if you are prompted to select a Python kernel, select *Python 3*.

The above notebooks will output the data reported in our paper.
Additionally, they produce our plots and if you execute the notebooks, the plots are saved as PDF files in the current directory.

Project architecture
--------------------

As a help to find your way around the codebase, here is a brief description of each file:

- `ppl.py` includes the probabilistic programming primitives as methods of the `ProbCtx` (probabilistic context) class
- `infer.py` implements NP-DHMC (and importance sampling for illustration) used in our experiments.
- `evaluation.py` contains utilities needed for the evaluation of the results.
- `example_*.py` are for running the experiments from the paper.
- `evaluation_*.ipynb` are for evaluating the results of each experiment.
